@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -23,7 +23,7 @@ Animation and VFX Studios.
 See docs for more information.
 """
 
-__version__ = '0.2.3.5'
+__version__ = '0.2.5'
 
 
 # before anything about stalker create the defaults
@@ -36,11 +36,13 @@ from stalker.models.entity import SimpleEntity, Entity
 from stalker.models.format import ImageFormat
 from stalker.models.link import Link
 from stalker.models.message import Message
-from stalker.models.mixins import (ProjectMixin, ReferenceMixin, ScheduleMixin,
-                                   StatusMixin, TargetEntityTypeMixin,
-                                   CodeMixin, WorkingHoursMixin)
+from stalker.models.mixins import (ProjectMixin, ReferenceMixin,
+                                   DateRangeMixin, StatusMixin,
+                                   TargetEntityTypeMixin, CodeMixin,
+                                   WorkingHoursMixin, ScheduleMixin)
 from stalker.models.note import Note
 from stalker.models.project import Project
+from stalker.models.review import Review
 from stalker.models.repository import Repository
 from stalker.models.scene import Scene
 from stalker.models.schedulers import SchedulerBase, TaskJugglerScheduler
@@ -50,7 +52,7 @@ from stalker.models.status import Status, StatusList
 from stalker.models.structure import Structure
 from stalker.models.studio import Studio, WorkingHours, Vacation
 from stalker.models.tag import Tag
-from stalker.models.task import TimeLog, Task
+from stalker.models.task import TimeLog, Task, TaskDependency
 from stalker.models.template import FilenameTemplate
 from stalker.models.ticket import Ticket, TicketLog
 from stalker.models.type import Type, EntityType
