@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -47,10 +47,7 @@ def setup(settings=None):
         "sqlalchemy" and shows the settings. The most important one is the
         engine. The default is None, and in this case it uses the settings from
         stalker.config.Config.database_engine_settings
-
-    :param callback: A callback function which is called after database is
-        initialized. It is a good place to register your own classes.
-   """
+    """
 
     if settings is None:
         settings = defaults.database_engine_settings
@@ -73,6 +70,22 @@ def setup(settings=None):
     logger.debug("creating the tables")
     Base.metadata.create_all(engine)
 
+    # update defaults
+    update_defaults_with_studio()
+
+
+def update_defaults_with_studio():
+    """updates the default values from Studio instance if a database and a
+    Studio instance is present
+    """
+    if DBSession:
+        with DBSession.no_autoflush:
+            from stalker.models.studio import Studio
+            studio = Studio.query.first()
+            if studio:
+                logger.debug('found a studio, updating defaults')
+                studio.update_defaults()
+
 
 def init():
     """fills the database with default values
@@ -83,9 +96,9 @@ def init():
     class_names = [
         'Asset', 'TimeLog', 'Department', 'Entity', 'FilenameTemplate',
         'Group', 'ImageFormat', 'Link', 'Message', 'Note', 'Permission',
-        'Project', 'Repository', 'Scene', 'Sequence', 'Shot', 'SimpleEntity',
-        'Status', 'StatusList', 'Structure', 'Studio', 'Tag', 'Task', 'Ticket',
-        'TicketLog', 'Type', 'User', 'Vacation', 'Version']
+        'Project', 'Repository', 'Review', 'Scene', 'Sequence', 'Shot',
+        'SimpleEntity', 'Status', 'StatusList', 'Structure', 'Studio', 'Tag',
+        'Task', 'Ticket', 'TicketLog', 'Type', 'User', 'Vacation', 'Version']
 
     for class_name in class_names:
         _temp = __import__(
@@ -102,8 +115,23 @@ def init():
     if defaults.auto_create_admin:
         __create_admin__()
 
-    # create Ticket statuses
-    __create_ticket_statuses()
+    # create statuses
+    create_ticket_statuses()
+    create_entity_statuses(entity_type='Task',
+                           status_names=defaults.task_status_names,
+                           status_codes=defaults.task_status_codes)
+    create_entity_statuses(entity_type='Asset',
+                           status_names=defaults.task_status_names,
+                           status_codes=defaults.task_status_codes)
+    create_entity_statuses(entity_type='Shot',
+                           status_names=defaults.task_status_names,
+                           status_codes=defaults.task_status_codes)
+    create_entity_statuses(entity_type='Sequence',
+                           status_names=defaults.task_status_names,
+                           status_codes=defaults.task_status_codes)
+    create_entity_statuses(entity_type='Review',
+                           status_names=defaults.review_status_names,
+                           status_codes=defaults.review_status_codes)
 
     logger.debug('finished initializing the database')
 
@@ -172,57 +200,18 @@ def __create_admin__():
     DBSession.commit()
 
 
-def __create_ticket_statuses():
+def create_ticket_statuses():
     """creates the default ticket statuses
     """
     from stalker import User
 
+    # create as admin
     admin = User.query.filter(User.login == defaults.admin_name).first()
 
     # create statuses for Tickets
-    from stalker import Status, StatusList
-
-    logger.debug("Creating Ticket Statuses")
-
-    # Update: check before adding
-    ticket_names = map(str.title, defaults.ticket_status_order)
-    ticket_statuses = Status.query.filter(Status.name.in_(ticket_names)).all()
-    if not ticket_statuses:
-        logger.debug('No Ticket Statuses found, creating new!')
-        ticket_statuses = [
-            Status(
-                name=status_name.title(),
-                code=status_name.upper(),
-                created_by=admin,
-                updated_by=admin
-            ) for status_name in defaults.ticket_status_order
-        ]
-    else:
-        logger.debug('Ticket Statuses are already created')
-
-    ticket_status_list = StatusList.query\
-        .filter(StatusList._target_entity_type=='Ticket')\
-        .first()
-
-    if not ticket_status_list:
-        logger.debug('No Ticket Status List found, creating new!')
-        ticket_status_list = StatusList(
-            name='Ticket Statuses',
-            target_entity_type='Ticket',
-            statuses=ticket_statuses,
-            created_by=admin,
-            updated_by=admin
-        )
-        DBSession.add(ticket_status_list)
-        try:
-            DBSession.commit()
-        except IntegrityError:
-            DBSession.rollback()
-        else:
-            logger.debug("Created Ticket Statuses successfully")
-            DBSession.flush()
-    else:
-        logger.debug("Ticket Status List already created")
+    ticket_names = defaults.ticket_status_names
+    ticket_codes = defaults.ticket_status_codes
+    create_entity_statuses('Ticket', ticket_names, ticket_codes, admin)
 
     # Again I hate doing this in this way
     from stalker import Type
@@ -263,20 +252,79 @@ def __create_ticket_statuses():
         logger.debug("Ticket Types are created successfully")
 
 
+def create_entity_statuses(entity_type='', status_names=None,
+                           status_codes=None, user=None):
+    """creates the default task statuses
+    """
+    if not entity_type:
+        raise ValueError('Please supply entity_type')
+
+    if not status_names:
+        raise ValueError('Please supply status names')
+
+    if not status_codes:
+        raise ValueError('Please supply status codes')
+
+    # create statuses for entity
+    from stalker import Status, StatusList
+
+    logger.debug("Creating %s Statuses" % entity_type)
+
+    statuses = Status.query.filter(Status.name.in_(status_names)).all()
+    status_names_in_db = map(lambda x: x.name, statuses)
+
+    for name, code in zip(status_names, status_codes):
+        if name not in status_names_in_db:
+            logger.debug('Creating Status: %s (%s)' % (name, code))
+            new_status = Status(
+                name=name,
+                code=code,
+                created_by=user,
+                updated_by=user
+            )
+            statuses.append(new_status)
+            DBSession.add(new_status)
+
+    # create the Status List
+    status_list = StatusList.query\
+        .filter(StatusList.target_entity_type == entity_type)\
+        .first()
+
+    if not status_list:
+        logger.debug('No %s Status List found, creating new!' % entity_type)
+        status_list = StatusList(
+            name='%s Statuses' % entity_type,
+            target_entity_type=entity_type,
+            statuses=statuses,
+            created_by=user,
+            updated_by=user
+        )
+        DBSession.add(status_list)
+    else:
+        logger.debug("%s Status List already created" % entity_type)
+
+    try:
+        DBSession.commit()
+    except IntegrityError:
+        DBSession.rollback()
+    else:
+        logger.debug("Created %s Statuses successfully" % entity_type)
+        DBSession.flush()
+
+
 def register(class_):
     """Registers the given class to the database.
 
-    It is mainly used to create the :class:`~stalker.models.auth.Action`\ s
-    needed for the :class:`~stalker.models.auth.User`\ s and
-    :class:`~stalker.models.auth.Group`\ s to be able to interact with the
+    It is mainly used to create the :class:`.Action`\ s needed for the
+    :class:`.User`\ s and :class:`.Group`\ s to be able to interact with the
     given class. Whatever class you have created needs to be registered.
 
     Example, lets say that you have a data class which is specific to your
     studio and it is not present in Stalker Object Model (SOM), so you need to
     extend SOM with a new data type. Here is a simple Data class inherited from
-    the :class:`~stalker.models.entity.SimpleEntity` class (which is the
-    simplest class you should inherit your classes from or use more complex
-    classes down to the hierarchy)::
+    the :class:`.SimpleEntity` class (which is the simplest class you should
+    inherit your classes from or use more complex classes down to the
+    hierarchy)::
 
       from sqlalchemy import Column, Integer, ForeignKey
       from stalker.models.entity import SimpleEntity
@@ -293,20 +341,18 @@ def register(class_):
                             primary_key=True)
 
     Now because Stalker is using Pyramid authorization mechanism it needs to be
-    able to have an :class:`~stalker.models.auth.Permission` about your new
-    class, so you can assign this :class;`~stalker.models.auth.Permission` to
-    your :class:`~stalker.models.auth.User`\ s or
-    :class:`~stalker.models.auth.Group`\ s. So you ned to register your new
-    class with stalker.db.register like shown below::
+    able to have an :class:`.Permission` about your new class, so you can
+    assign this :class;`.Permission` to your :class:`.User`\ s or
+    :class:`.Group`\ s. So you ned to register your new class with
+    :func:`stalker.db.register` like shown below::
 
       from stalker import db
       db.register(MyDataClass)
 
     This will create the necessary Actions in the 'Actions' table on your
-    database, then you can create :class:`~stalker.models.auth.Permission`\ s
-    and assign these to your :class:`~stalker.models.auth.User`\ s and
-    :class:`~stalker.models.auth.Group`\ s so they are Allowed or Denied to do
-    the specified Action.
+    database, then you can create :class:`.Permission`\ s and assign these to
+    your :class:`.User`\ s and :class:`.Group`\ s so they are Allowed or Denied
+    to do the specified Action.
 
     :param class_: The class itself that needs to be registered.
     """
@@ -316,10 +362,11 @@ def register(class_):
     permissions_db = Permission.query.all()
 
     if not isinstance(class_, type):
-        raise TypeError('To register a class please supply the class itself ')
+        raise TypeError('To register a class please supply the class itself.')
 
     # register the class name to entity_types table
-    from stalker import EntityType, StatusMixin, ScheduleMixin, ReferenceMixin
+    from stalker import (EntityType, StatusMixin, DateRangeMixin,
+                         ReferenceMixin, ScheduleMixin)
 
     class_name = class_.__name__
 
@@ -328,6 +375,8 @@ def register(class_):
         # update attributes
         if issubclass(class_, StatusMixin):
             new_entity_type.statusable = True
+        if issubclass(class_, DateRangeMixin):
+            new_entity_type.dateable = True
         if issubclass(class_, ScheduleMixin):
             new_entity_type.schedulable = True
         if issubclass(class_, ReferenceMixin):
