@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Stalker a Production Asset Management System
-# Copyright (C) 2009-2013 Erkan Ozgur Yilmaz
+# Copyright (C) 2009-2014 Erkan Ozgur Yilmaz
 #
 # This file is part of Stalker.
 #
@@ -19,13 +19,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import unittest2
-from stalker import (Link, Project, Repository, Asset, Sequence, Shot, Status,
-                     StatusList, Task, Type, Version, FilenameTemplate,
-                     Structure)
-from stalker import db
 from stalker.db.session import DBSession
-
-from stalker import config
+from stalker import (db, config, Link, Project, Repository, Asset, Sequence,
+                     Shot, Status, StatusList, Task, Type, Version, Structure,
+                     FilenameTemplate)
 from stalker.exceptions import CircularDependencyError
 
 defaults = config.Config()
@@ -46,14 +43,12 @@ class VersionTester(unittest2.TestCase):
         """setting up the test in class level
         """
         DBSession.remove()
-        DBSession.configure(extension=None)
 
     @classmethod
     def tearDownClass(cls):
         """clean up the test in class level
         """
         DBSession.remove()
-        DBSession.configure(extension=None)
 
     def setUp(self):
         """setup the test
@@ -853,12 +848,24 @@ class VersionTester(unittest2.TestCase):
             'tp/SH001/Task1'
         )
 
-        new_version1.extension = '.ma'
+        extension = '.ma'
+        new_version1.extension = extension
         self.assertEqual(
             new_version1.filename,
             'Task1_TestTake_v001.ma'
         )
 
+        # rename the task and update the paths
+        self.test_task1.name = 'Task2'
+
+        # now call update_paths and expect the extension to be preserved
+        new_version1.update_paths()
+        self.assertEqual(
+            new_version1.filename,
+            'Task2_TestTake_v001.ma'
+        )
+        self.assertEqual(new_version1.extension, extension)
+
     def test_update_paths_will_raise_a_RuntimeError_if_there_is_no_suitable_FilenameTemplate(self):
         """testing if update_paths method will raise a RuntimeError if there is
         no suitable FilenameTemplate instance found
